@@ -2,7 +2,7 @@
 
 __author__ = """Qiusheng Wu"""
 __email__ = "giswqs@gmail.com"
-__version__ = "0.20.4"
+__version__ = "0.36.2"
 
 import os
 
@@ -52,9 +52,11 @@ else:
             print(
                 "Please restart Jupyter kernel after installation if you encounter any errors when importing geemap."
             )
-        raise Exception(e)
+        raise e
 
 if _use_eerepr():
     import eerepr
 
+    eerepr.initialize()
+
 from .report import Report
