@@ -1,3 +1,11 @@
+"""The cartoee module contains functions for creating publication-quality maps with cartopy and Earth Engine data."""
+
+# *******************************************************************************#
+# This module contains extra features of the geemap package.                     #
+# The geemap community will maintain the extra features.                         #
+# *******************************************************************************#
+
+
 import logging
 import os
 import subprocess
@@ -15,7 +23,7 @@ import requests
 from matplotlib import cm, colors
 from matplotlib import font_manager as mfonts
 
-from .basemaps import xyz_tiles
+from .basemaps import custom_tiles
 
 try:
     import cartopy.crs as ccrs
@@ -160,7 +168,9 @@ def get_map(ee_object, proj=None, basemap=None, zoom_level=2, **kwargs):
     if basemap is not None:
         if isinstance(basemap, str):
             if basemap.upper() in ["ROADMAP", "SATELLITE", "TERRAIN", "HYBRID"]:
-                basemap = cimgt.GoogleTiles(url=xyz_tiles[basemap.upper()]["url"])
+                basemap = cimgt.GoogleTiles(
+                    url=custom_tiles["xyz"][basemap.upper()]["url"]
+                )
 
         try:
             ax.add_image(basemap, zoom_level)
@@ -838,7 +848,7 @@ def add_scale_bar(
         xcoords.append(dx)
         ycoords.append(ymean)
 
-    # Convertin to arrays:
+    # Converting to arrays:
     xcoords = np.asanyarray(xcoords)
     ycoords = np.asanyarray(ycoords)
 
@@ -1149,7 +1159,11 @@ def get_image_collection_gif(
     file_format="png",
     north_arrow_dict={},
     scale_bar_dict={},
+    overlay_layers=[],
+    overlay_styles=[],
+    colorbar_dict={},
     verbose=True,
+    **kwargs,
 ):
     """Download all the images in an image collection and use them to generate a gif/video.
     Args:
@@ -1165,10 +1179,14 @@ def get_image_collection_gif(
         date_format (str, optional): A pattern, as described at http://joda-time.sourceforge.net/apidocs/org/joda/time/format/DateTimeFormat.html. Defaults to "YYYY-MM-dd".
         fig_size (tuple, optional): Size of the figure.
         dpi_plot (int, optional): The resolution in dots per inch of the plot.
-        file_format (str, optional): Either 'png' or 'jpg'.
+        file_format (str, optional): Either 'png' or 'jpg'. Defaults to 'png'.
         north_arrow_dict (dict, optional): Parameters for the north arrow. See https://geemap.org/cartoee/#geemap.cartoee.add_north_arrow. Defaults to {}.
         scale_bar_dict (dict, optional): Parameters for the scale bar. See https://geemap.org/cartoee/#geemap.cartoee.add_scale_bar. Defaults. to {}.
+        overlay_layers (list, optional): A list of Earth Engine objects to overlay on the map. Defaults to [].
+        overlay_styles (list, optional): A list of dictionaries of visualization parameters for overlay layers. Defaults to [].
+        colorbar_dict (dict, optional): Parameters for the colorbar. See https://geemap.org/cartoee/#geemap.cartoee.add_colorbar. Defaults to {}.
         verbose (bool, optional): Whether or not to print text when the program is running. Defaults to True.
+        **kwargs: Additional keyword arguments are passed to the add_layer() function.
     """
 
     from .geemap import png_to_gif, jpg_to_gif
@@ -1209,6 +1227,47 @@ def get_image_collection_gif(
         # Plot image
         ax = get_map(image, region=region, vis_params=vis_params, cmap=cmap, proj=proj)
 
+        # check length of overlay layers and styles
+        if len(overlay_layers) != len(overlay_styles):
+            raise ValueError(
+                "The length of overlay_layers and overlay_styles must be the same."
+            )
+
+        for ee_object, style in zip(overlay_layers, overlay_styles):
+            if (
+                isinstance(ee_object, ee.geometry.Geometry)
+                or isinstance(ee_object, ee.feature.Feature)
+                or isinstance(ee_object, ee.featurecollection.FeatureCollection)
+            ):
+                overlay_vis_params = (
+                    None  # for vector data, we can pass style parameters directly
+                )
+            elif (
+                isinstance(ee_object, ee.image.Image)
+                or isinstance(ee_object, ee.imagecollection.ImageCollection)
+                or isinstance(ee_object, ee.imagecollection.ImageCollection)
+            ):
+                overlay_vis_params = style  # for raster, we need to pass vis_params
+                style = None
+            else:
+                raise ValueError(
+                    "The overlay object must be an ee.Geometry, ee.Feature, ee.FeatureCollection, ee.Image, or ee.ImageCollection."
+                )
+
+            add_layer(
+                ax,
+                ee_object,
+                region=region,
+                cmap=cmap,
+                vis_params=overlay_vis_params,
+                style=style,
+                **kwargs,
+            )
+
+        # Add colorbar if colorbar_dict is not empty
+        if colorbar_dict:
+            add_colorbar(ax, vis_params, **colorbar_dict)
+
         # Add grid
         if grid_interval is not None:
             add_gridlines(ax, interval=grid_interval, linestyle=":")
