@@ -1,3 +1,10 @@
+"""Module for creating interactive maps with kepler.gl."""
+
+# *******************************************************************************#
+# This module contains extra features of the geemap package.                     #
+# The geemap community will maintain the extra features.                         #
+# *******************************************************************************#
+
 import json
 import os
 import sys
