@@ -1,5 +1,9 @@
-"""Module of sample legends for some commonly used geospatial datasets.
-"""
+"""Module of sample legends for some commonly used geospatial datasets."""
+
+# *******************************************************************************#
+# This module contains extra features of the geemap package.                     #
+# The geemap community will maintain the extra features.                         #
+# *******************************************************************************#
 
 import os
 
@@ -488,8 +492,6 @@ def ee_table_to_legend(in_table, out_file):
         in_table (str): The input file path (*.txt) to the Earth Engine color table.
         out_file (str): The output file path (*.txt) to the legend dictionary.
     """
-    # pkg_dir = os.path.dirname(pkg_resources.resource_filename("geemap", "geemap.py"))
-    # ee_legend_table = os.path.join(pkg_dir, "data/template/ee_legend_table.txt")
 
     if not os.path.exists(in_table):
         print("The class table does not exist.")
