@@ -1,3 +1,10 @@
+"""Module for creating interactive maps with plotly."""
+
+# *******************************************************************************#
+# This module contains extra features of the geemap package.                     #
+# The geemap community will maintain the extra features.                         #
+# *******************************************************************************#
+
 import os
 import numpy as np
 import pandas as pd
@@ -63,8 +70,8 @@ class Canvas:
 
     def toolbar_reset(self):
         """Reset the toolbar so that no tool is selected."""
-        if hasattr(self, "toolbar"):
-            toolbar_grid = self.toolbar
+        if hasattr(self, "_toolbar"):
+            toolbar_grid = self._toolbar
             for tool in toolbar_grid.children:
                 tool.value = False
 
@@ -502,7 +509,7 @@ class Map(go.FigureWidget):
         """Adds a COG TileLayer to the map.
 
         Args:
-            url (str): The URL of the COG tile layer, e.g., 'https://opendata.digitalglobe.com/events/california-fire-2020/pre-event/2018-02-16/pine-gulch-fire20/1030010076004E00.tif'
+            url (str): The URL of the COG tile layer, e.g., 'https://github.com/opengeos/data/releases/download/raster/Libya-2023-07-01.tif'
             name (str, optional): The layer name to use for the layer. Defaults to 'Untitled'.
             attribution (str, optional): The attribution to use. Defaults to ''.
             opacity (float, optional): The opacity of the layer. Defaults to 1.
@@ -710,7 +717,7 @@ class Map(go.FigureWidget):
             **kwargs,
         )
 
-        self.add_basemap("Stamen.Terrain")
+        self.add_basemap("Esri.WorldTopoMap")
         self.add_trace(heatmap)
 
     def add_gdf(
